const { EmbedBuilder } = require("discord.js");
const Queue = require("@src/database/schemas/Queue");
const { MUSIC } = require("@root/config");

module.exports = {
  name: "join",
  description: "Join the voice channel and resume playback if there is a saved queue.",
  category: "MUSIC",
  validations: [], // Add any validations if necessary
  command: {
    enabled: true,
  },
  slashCommand: {
    enabled: true,
  },

  async messageRun(message, args) {
    const response = await executeJoinCommand(message);
    await message.safeReply(response);
  },

  async interactionRun(interaction) {
    const response = await executeJoinCommand(interaction);
    await interaction.followUp(response);
  },
};

async function executeJoinCommand(interaction) {
  const { guild, member } = interaction;
  const voiceChannel = member.voice.channel;

  if (!voiceChannel) {
    return "You need to be in a voice channel to use this command.";
  }

  // Get or create the player for the guild
  let player = guild.client.musicManager.getPlayer(guild.id);
  if (!player) {
    player = await guild.client.musicManager.createPlayer({
      guildId: guild.id,
      voiceChannelId: voiceChannel.id,
      textChannelId: interaction.channel.id,
      selfMute: false,
      selfDeaf: true,
      volume: MUSIC.DEFAULT_VOLUME,
    });
  }

  if (!player.connected) await player.connect()

  // Check for saved queue in the database
  const restore = await Queue.restore(player)

  if (restore) {
    await player.play({ paused: false });

    return `Joined ${voiceChannel.name} and resumed playback of **${player.queue.current.info.title}**.`;
  } else {
    return `Joined ${voiceChannel.name}.`
  }
}