const { musicValidations } = require("@helpers/BotUtils");

/**
 * @type {import("@structures/Command")}
 */
module.exports = {
  name: "skip",
  description: "Skip the current song",
  category: "MUSIC",
  validations: musicValidations,
  command: {
    enabled: true,
    aliases: ["next", "s"],
  },
  slashCommand: {
    enabled: true,
  },

  async messageRun(message, args) {
    const response = await skip(message);
    await message.safeReply(response);
  },

  async interactionRun(interaction) {
    const response = await skip(interaction);
    await interaction.followUp(response);
  },
};

/**
 * @param {import("discord.js").CommandInteraction|import("discord.js").Message} arg0
 */
async function skip({ client, guildId }) {
  const player = client.musicManager.getPlayer(guildId);

  if (!player || !player.queue.current) {
    return "🚫 There's no music currently playing";
  }

  const title = player.queue.current.info.title;

  // Check if the node is connected
  const node = player.node;
  if (!node || !node.connected) {
    try {
      await node.connect(); // Attempt to connect the node
    } catch (error) {
      console.error(`Failed to connect to Lavalink node: ${error.message}`);
      return "🚫 Failed to connect to the music server. Please try again later.";
    }
  }

  // Proceed to skip the track
  try {
    await player.skip();
    return `⏯️ ${title} was skipped`;
  } catch (error) {
    console.error(`Error while skipping track: ${error.message}`);
    return "🚫 An error occurred while trying to skip the track.";
  }
}
