const path = require('path')
const { ApplicationCommandOptionType, ApplicationCommandType } = require('discord.js')

/**
 * @type {import("@structures/Command")}
 */
module.exports = {
  name: "reloadcmd",
  description: "Reloads commands",
  category: "OWNER",
  command: {
    enabled: process.env.DEV === 'true',
    usage: "<filename> [guild]",
    minArgsCount: 1,
  },
  slashCommand: {
    enabled: process.env.DEV === 'true',
    options: [
      {
        name: "filename",
        description: "File name to load. Reloads all if not specified",
        type: ApplicationCommandOptionType.String,
        required: true,
      }, {
        name: "guild",
        description: "guild",
        type: ApplicationCommandOptionType.String,
        required: false,
      }, {
        name: "reregister",
        description: "Reregisters command to manager",
        type: ApplicationCommandOptionType.Boolean,
        required: false,
      },
    ],
  },

  async messageRun(message, args) {
    await message.safeReply(await runReload(message.client, args[0], args[1]))
  },

  async interactionRun(interaction) {
    await interaction.followUp(await runReload(
      interaction.client,
      interaction.options.getString("filename"),
      interaction.options.getString("guild"),
      interaction.options.getBoolean("reregister"),
    ))
  }
};

async function runReload(client, filename, guildid, rereg = false) {
  // guild id
  let guild
  if (guildid) {
    guild = client.guilds.cache.get(guildid);
    if (!guild) return "No matching guild"
  }

  // load
  if (filename[0] === '/') {
    const cmdname = filename.slice(1)
    const cmd = client.commands.get(cmdname)
    if (!cmd) return "Cannot find command with name " + cmd
    if (!cmd.category) return `Command category ${cmd} is undefined`

    filename = cmd.category.toLowerCase() + '/' + cmdname
  }

  if (!path.extname(filename)) filename += '.js'
  const target = path.resolve('src', 'commands', path.normalize(filename))
  const oldCmd = require.cache[target]?.exports
  const oldSlashEnable = oldCmd?.slashCommand?.enabled
  delete require.cache[target]
  const newCmd = require(target)
  const newSlashEnable = newCmd?.slashCommand?.enabled

  if (rereg) {
    // register / edit
    const cmdManager = guild ? guild.commands : client.application.commands
    const slashCmd = {
      name: newCmd.name,
      description: newCmd.description,
      type: ApplicationCommandType.ChatInput,
      options: newCmd.slashCommand?.options
    }

    // slash commands
    if (oldSlashEnable) {
      const oldSlash = client.application.commands.cache.find(v => v.name === oldCmd.name)
      if (oldSlash) {
        if (newSlashEnable) await cmdManager.edit(oldSlash, slashCmd)
        else await cmdManager.delete(oldSlash)
      }
    }
    else if (newSlashEnable) {
      await cmdManager.create(slashCmd)
    }
  }

  // remove old
  if (oldCmd?.command?.enabled) {
    client.commands.delete(oldCmd.name.toLowerCase())
    for (const alias of oldCmd.command.aliases ?? []) client.commandAlias.delete(alias)
  }
  if (oldSlashEnable) {
    client.slashCommands.delete(oldCmd.name);
  }

  // add new
  if (newCmd?.command?.enabled) {
    client.commands.set(newCmd.name.toLowerCase(), newCmd)
    for (const alias of newCmd.command.aliases ?? []) client.commandAlias.set(alias, newCmd)
  }
  if (newSlashEnable) {
    client.slashCommands.set(newCmd.name, newCmd);
  }

  return `Reloaded ${filename}`
}
