//@ts-check
const queue = require("@root/src/commands/music/queue");
const mongoose = require("mongoose");

const queueSchema = new mongoose.Schema({
  guildId: { type: String, required: true, unique: true },
  tracks: [
    {
      title: String,
      uri: String,
      duration: Number,
      requester: String,
    },
  ],
  currentTrack: {
    title: String,
    uri: String,
    duration: Number,
    requester: String,
  },
});

const QueueModel = mongoose.model("Queue", queueSchema)

module.exports = {
  model: QueueModel,

  async save(player) {
    const queue = player.queue
    const currentQueue = queue.current

    const queueData = {
      guildId: player.guildId,
      tracks: queue.tracks.map(track => ({
        title: track.info.title,
        uri: track.info.uri,
        duration: track.info.duration,
        requester: track.requester.id,
      })),
      currentTrack: currentQueue?.info?.title ? {
        title: currentQueue.info.title,
        uri: currentQueue.info.uri,
        duration: currentQueue.info.duration,
        requester: currentQueue.requester.id,
      } : null,
    };

    await QueueModel.findOneAndUpdate(
      { guildId: player.guildId },
      queueData,
      { upsert: true, new: true }
    )
  },

  async saveCurrentTrack(player, track) {
    const queueData = {
      guildId: player.guildId,
      currentTrack: {
        title: track.info.title,
        uri: track.info.uri,
        duration: track.info.duration,
        requester: track.requester.id,
        position: player.position || 0, // Save the current position
      },
    };

    await QueueModel.findOneAndUpdate(
      { guildId: player.guildId },
      queueData,
      { upsert: true, new: true }
    );
  },

  async findGuild(guildOrPlayer) {
    if ('guildId' in guildOrPlayer) guildOrPlayer = guildOrPlayer.guildId
    return QueueModel.findOne({ guildId: guildOrPlayer });
  },

  async deleteGuild(guildOrPlayer) {
    if ('guildId' in guildOrPlayer) guildOrPlayer = guildOrPlayer.guildId
    return QueueModel.deleteOne({ guildId: guildOrPlayer });
  },

  /**
   * @param {import('lavalink-client').Player} player
   * @returns
   */
  async restore(player) {
    const queueData = await this.findGuild(player)
    if (!queueData) return false
    const { tracks, currentTrack } = queueData

    player.queue.utils.save()

    player.queue.add(tracks.map(track => ({
      info: {
        title: track.title,
        uri: track.uri,
        duration: track.duration,
      },
      requester: { id: track.requester },
    })));

    if (currentTrack?.uri) {
      player.queue.current = {
        info: {
          title: currentTrack.title,
          uri: currentTrack.uri,
          duration: currentTrack.duration,
        },
        requester: { id: currentTrack.requester },
      };

      // Set the current position if available
      player.position = currentTrack.position || 0; // Default to 0 if not set
    }

    return true
  }
};
